/*
******************************************************************************

   LibBLT: extension API header file

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]
   Copyright (C) 2001 Brian S. Julin    [bri@tull.umassp.edu]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#ifndef _GGI_BLT_H
#define _GGI_BLT_H

#ifndef _GGI_INTERNAL_GII_H
#include <ggi/ggi.h>
#endif

__BEGIN_DECLS

#include <ggi/galloc.h>
#include <ggi/mmutil.h>

/* Extension management
 */
int ggiBltInit(void);
int ggiBltExit(void);

int ggiBltAttach(ggi_visual_t vis, ggiGA_resource_list *reqlist);
int ggiBltDetach(ggi_visual_t vis);

/* Structures
 */
#ifdef HAVE_STRUCT_BLT
typedef struct ggiBlt *ggiBlt_t;
#else
struct ggiBlt { };

typedef struct ggiBlt *ggiBlt_t;
#endif

/* batchop interface */
#define GGI_OP_OWNER_BUF	0x00010000	/* Our opcode registry entry */

#define BLT_OP_SIMPLEBLT	0x00010000
/* We use a flag system, any combination of the below us a unique opcode */
#define BLT_OP_ROPBLT		0x00010001	/* Blt uses a boolean ROP */
#define BLT_OP_KEYBLT		0x00010002	/* Blt uses a color key */
#define BLT_OP_ALPHABLT		0x00010004	/* Blt uses an alpha channel */
#define BLT_OP_ZBLT		0x00010008	/* Blt uses a Z channel */
#define BLT_OP_BUFBLT		0x00010010	/* source/dest is ggiBuf_t */
#define BLT_OP_PAT		0x00010020	/* Blt tesslates */
#define BLT_OP_SRC2		0x00010040      /* Multi-source blit */
/* TODO: This list is not complete for multi-source */

/* Parmtypes */
#define BLT_PT_DSTX (GGI_PT_IS_COORD | GGI_PT_IN_X | GGI_PT_ROLE_START)
#define BLT_PT_SRCX (BLT_PT_DSTX | GGI_PT_IS_SRC)
#define BLT_PT_SRCW \
(GGI_PT_IS_COORD | GGI_PT_IN_X | GGI_PT_ROLE_LEN | GGI_PT_IS_SRC)

#define BLT_PT_DSTY (GGI_PT_IS_COORD | GGI_PT_IN_Y | GGI_PT_ROLE_START)
#define BLT_PT_SRCY (BLT_PT_DSTY | GGI_PT_IS_SRC)
#define BLT_PT_SRCH \
(GGI_PT_IS_COORD | GGI_PT_IN_Y | GGI_PT_ROLE_LEN | GGI_PT_IS_SRC)

#define BLT_PT_SRCADDR (GGI_PT_IN_ADDR | GGI_PT_ROLE_START | GGI_PT_IS_SRC)
#define BLT_PT_DSTADDR (GGI_PT_IN_ADDR | GGI_PT_ROLE_START)


/* See ggiBltCreate(3GGI) manpage */
enum ggiBlt_flags {
	BLT_FLAGS_STICKY_CLIP	= 0x00000001,
	BLT_FLAGS_STICKY_FGBG	= 0x00000002,
	BLT_FLAGS_STICKY_BLEND	= 0x00000004
};

/* To be documented in LibBlt structure-releated documentation */
enum ggiBlt_source_flags {
	BLT_FLAGS_SRC1_MASK	= 0x0000000f,
	BLT_FLAGS_SRC1_SHIFT	= 0,
	BLT_FLAGS_SRC2_MASK 	= 0x000000f0,
	BLT_FLAGS_SRC2_SHIFT	= 8,
	BLT_FLAGS_DST_MASK	= 0x00000f00,
	BLT_FLAGS_DST_SHIFT	= 16,
	BLT_FLAGS_BOB		= 0x0,
	BLT_FLAGS_GGIVIS 	= 0x1,
	BLT_FLAGS_BUF		= 0x2,
	BLT_FLAGS_NESTED        = 0x3,
	BLT_FLAGS_ISLIST	= 0x10000000, /* Use .next */
};

/* See the ggiBltSetBobFlags(3GGI) manpage */
enum ggiBlt_bob_flags {
	BLT_BOBFLAGS_KEYNOT = 0x00000001
};

struct ggiBlt_source;
typedef struct ggiBlt_source *ggiBlt_source_t;
typedef struct ggiBlt_source *ggiBlt_source_list;

struct ggiBlt_source { /* Yes, this is a public, non-opaque structure. */
	enum ggiBlt_source_flags flags;

	/* Pointer to operands -- flags determine type */
        void *src1;  /* Do NOT touch when this source is LibBlt-native */
        void *src2;  /* This one OK to touch */

	/* Pointer to dest (NULL == main visual framebuffer of ggiBlt_t),
	 * otherwise flags determine type. 
	 */
	void *dst;
	ggiBlt_source_list next;
};

/* API functions
 */

/* LibGAlloc lazy interface */

ggiBlt_t ggiBltCreate(ggi_visual_t vis, enum ggiGA_storage_type stype,
		      int numrows, enum ggiGA_rop rop, 
		      enum ggiBlt_flags flags);
int ggiBltDestroy(ggiBlt_t *blt);

ggiBlt_source_t ggiBltCreateBob(ggi_visual_t vis, ggiBlt_t forthis,
				enum ggiGA_storage_type stype, int w, int h,
				int qty, ggi_graphtype gt);
int ggiReuseBob(ggiBlt_source_t bob, ggiBlt_t forthis);
int ggiBltDestroyBob(ggiBlt_source_t *blt);

/* LibGAlloc advanced interface.
 */
int ggiBltAdd(ggiGA_resource_list *reqlist,
              struct ggiGA_resource_props **props, /* motor, bobs, carbs */
              enum ggiGA_resource_type *types,
              int numresources,
              ggiGA_resource_handle **handles);
/* More convenient functions can be added as needed. */
ggiBlt_t ggiBltTie(ggiGA_resource_handle *handles, int numhandles);
int ggiBltUntie(ggiBlt_t *blt);
ggiBlt_t ggiBltTieBob(ggiGA_resource_handle *carb, 
		      ggiGA_resource_handle *tank);
int ggiBltUntieBob(ggiBlt_source_t *bob);

/* Functions for BOB based ggiBlt_source objects */
int ggiBltSetBobRop(ggiBlt_source_t bob, enum ggiGA_rop rop);
int ggiBltGetBobRop(ggiBlt_source_t bob, enum ggiGA_rop *rop);
int ggiBltSetBobKeyValue(ggiBlt_source_t bob, ggi_pixel keyval);
int ggiBltGetBobKeyValue(ggiBlt_source_t bob, ggi_pixel *keyval);
int ggiBltSetBobKeyMask(ggiBlt_source_t bob, ggi_pixel keymask);
int ggiBltGetBobKeyMask(ggiBlt_source_t bob, ggi_pixel *keymask);
int ggiBltSetBobFlags(ggiBlt_source_t bob, enum ggiBlt_bob_flags flags);
int ggiBltGetBobFlags(ggiBlt_source_t bob, enum ggiBlt_bob_flags *flags);

/* Functions that may work on buf/visual/other types of sources as well */
int ggiBltSetSourceIdx(ggiBlt_source_t bob, int idx);
int ggiBltGetSourceIdx(ggiBlt_source_t bob, int idx);

/* Blit "spreadsheet" interface */
int ggiBltSetIdx(ggiBlt_t blt, int idx);
int ggiBltGetIdx(ggiBlt_t blt, int *idx);
int ggiBltGo(ggiBlt_t blt, int start, int numrows);
int ggiBltAt(ggiBlt_t blt);

int ggiBltPutBatchop(ggiBlt_t into, batchop_t from, int numrows);

int ggiBltPutBlit(ggiBlt_t blt, ggiBlt_source_t source,
		  long x, long y, int w, int h);

int ggiBltPutReverseBlit(ggiBlt_t blt, ggiBlt_source_t dest,
			 long x, long y, int w, int h);

int ggiBltPutStretchBlit(ggiBlt_t blt, ggiBlt_source_t source,
			 long x, long y, int sw, int sh, long dw, long dh);

int ggiBltPutZBlit(ggiBlt_t blt, ggiBlt_source_t source,
		   long x, long y, int w, int h,
		   int ztl, int zbr);

int ggiBltPutABlit(ggiBlt_t blt, ggiBlt_source_t source,
		   long x, long y, int w, int h,
		   int atl, int abr);

int ggiBltAlterBobIdx(ggiBlt_t blt, int *bobidx, int numrows);
int ggiBltAlterCoord(ggiBlt_t blt, long *x, long *y, int numrows);
int ggiBltAlterBox(ggiBlt_t blt, long *x, long *y, int *w, int *h, 
		   int numrows);
int ggiBltAlterStretchBox(ggiBlt_t blt, long *x, long *y, int *w, int *h, 
			  int numrows);
int ggiBltAlterZ(ggiBlt_t blt, int *ztl, int *zbr, int numrows);
int ggiBltAlterA(ggiBlt_t blt, int *atl, int *abr, int numrows);

/* Other varients of the above convenience functions will follow. */

/* Set the default base for the coordinates: Either pixels or dots
 */
int ggiBltSetCoordBase(ggiBlt_t blt, enum ggiGA_coordbase cb);
enum ggiGA_coordbase ggiBltGetCoordBase(ggiBlt_t blt);

__END_DECLS

#endif /* _GGI_BLT_H */
